#include <USBKit.h>
#include <stdio.h>


#define USBPULSEBOX_VENDORID 0x????
#define USBPULSEBOX_PRODUCTID 0x????


extern "C" int  _kget_cpu_state_(int cpu);
extern "C" int  _kset_cpu_state_(int cpu, int enabled);


class TUSBRoster : public USBRoster
{
private:
	BList	 usb_devs;	// USBデバイスのリスト
public:
	virtual status_t	DeviceAdded(USBDevice *dev);
	virtual void		DeviceRemoved(USBDevice *dev);
};

USBDevice *d = NULL;

// USBデバイスが追加された時のイベント
status_t TUSBRoster::DeviceAdded(USBDevice *dev)
{
	printf("check %s : VID = 0x%x  PID = 0x%x\n",dev->Location(),dev->VendorID(),dev->ProductID());
	if ((dev->VendorID() == USBPULSEBOX_VENDORID) && (dev->ProductID() == USBPULSEBOX_PRODUCTID)) {
		d = dev;
		printf("found USB PulseBOX\n");
		return B_OK;
	} else {
		return B_ERROR;
	}
}

// USBデバイスが削除（抜かれた）ときのイベント
void TUSBRoster::DeviceRemoved(USBDevice *dev)
{
	printf("removed %s : VID = 0x%x  PID = 0x%x\n",dev->Location(),dev->VendorID(),dev->ProductID());
	d = NULL;
}

int16 Demultiplexer(unsigned char input)
{
	switch(input){
		case 0: return ~0;
		case 1: return ~0x1f;
		case 2: return ~0x1f;
		case 3: return ~0x1f;
		case 4: return ~0x1f;
		case 5: return ~0x1f;
		case 6: return ~0x3f;
		case 7: return ~0x7f;
		case 8: return ~0xff;
		case 9: return ~0x1ff;
		case 10: return ~0x3ff;
		case 11: return ~0x7ff;
		case 12: return ~0xfff;
		case 13: return ~0x1fff;
		case 14: return ~0x3fff;
		case 15: return ~0x7fff;
		default: return 0;
	}
}

int main(int argc, char *argv[])
{
	uchar data[8];
	int16 CPU[B_MAX_CPU_COUNT +1];
	int i;
	system_info sysinfo;
	bigtime_t now, PrevTick[B_MAX_CPU_COUNT], last = 0;

	int press1, press2;
	int last_pressed1 = 0;
	int last_pressed2 = 0;

	char btn = 0;

	TUSBRoster usb_roster;
	usb_roster.Start();

	while (d==NULL) snooze(1000);

//	usb_roster.Stop();

	d->SetConfiguration(d->ConfigurationAt(0));
	while (true) {
		now = system_time();
		get_system_info(&sysinfo);
		for(i=0;i<sysinfo.cpu_count;i++) {
			CPU[i] = 16 * (sysinfo.cpu_infos[i].active_time - PrevTick[i]) / (now-last);
			PrevTick[i] = sysinfo.cpu_infos[i].active_time;
		}
//		printf("%d\n",CPU[0]);
		last = now;

		btn=_kget_cpu_state_(0) + (2*_kget_cpu_state_(1));
		d->ControlTransfer(USB_REQTYPE_VENDOR|USB_REQTYPE_ENDPOINT_IN,
							btn,
							Demultiplexer(CPU[0]),
							Demultiplexer(CPU[1]),
							8,data);
//		printf("%2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x\n",data[0],data[1],data[2],data[3],data[4],data[5],data[6],data[7]);

		press1 = (~data[0]) & 1;
		press2 = (~data[0]) & 2;

//		printf("%2.2x %d %d\n",~data[0],press1,press2);

		if ((press1==1) && (press1!=last_pressed1)) {
			_kset_cpu_state_(0,!_kget_cpu_state_(0));
		}
		if ((press2==2) && (press2!=last_pressed2)) {
			_kset_cpu_state_(1,!_kget_cpu_state_(1));
		}

		last_pressed1 = press1;
		last_pressed2 = press2;

		snooze(100000);
	}
	return 0;
}
